// *****************************************************************************
// FORM OF LICENSE FOR APPWIZARD FILES

// TEXAS INSTRUMENTS TEXT FILE LICENSE
// FOR TAPKO APPWIZARD FILES TO BE USED WITH TAPKO KAISTACK

// Copyright (c) 2015 Texas Instruments Incorporated
// Portions Copyright (c) 1999 * 2009 Tapko Technologies GmbH

// All rights reserved not granted herein.

// Limited License.

// Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive license under
// copyrights and patents it now or hereafter owns or controls to make, have made, use, import,
// offer to sell and sell ("Utilize") this software subject to the terms herein. With respect to the
// foregoing patent license, such license is granted solely to the extent that any such patent is
// necessary to Utilize the software alone. The patent license shall not apply to any combinations
// which include this software, other than combinations with devices manufactured by or for TI ("TI
// Devices"). No hardware patent is licensed hereunder'

// Use for Evaluation, Development and Support.

// You may copy, have copied, display and have displayed internally, distribute internally and use
// and have used internally this software for the sole purposes of evaluating and testing the
// software and designing, developing, maintaining and supporting your products, provided that
// the following conditions are met:

// * No reverse engineering, decompilation, or disassembly of this software is permitted with
//   respect to any software provided in binary form.
// * Any use is licensed by TI for use only with TI Devices and the Tapko KAIstack software.
// * Nothing shall obligate TI to provide you with source code for the software licensed and
//   provided to you in object code.

// Redistributions.

// Redistributions, if permitted as provided below, must preserve existing copyright notices and reproduce
// this license (including the above copyright notices and the disclaimer and (if
// applicable) source code license limitations below) in the documentation and/or other materials
// provided with the distribution.

// Redistributions in Binary Form.

// Redistribution in binary form, without modification, is permitted provided that the following
// conditions are met:
// * No reverse engineering, decompilation, or disassembly of this software is permitted with
//   respect to any software provided in binary form.
// * Any redistribution and use are licensed by TI:
//   - for use only with TI Devices and the Tapko KAIstack software;
//   - only if you have first obtained a commercial license for the software from Tapko
//     Technologies GmbH; and
//   - subject to the license rights and restrictions in your commercial license for the software
//     from Tapko Technologies GmbH.
// * Nothing shall obligate TI to provide you with source code for the software licensed and
// provided to you in object code.

// Redistributions in Source Form or Compiled from Source.

// If software source code is provided to you, modification and redistribution of the source code are
// permitted provided that the following conditions are met:

// * Any redistribution and use of the source code, including any resulting derivative works, are
//   licensed by TI:
//   - for use only with TI Devices and the Tapko KAIstack software;
//   - only if you have first obtained a commercial license for the software from Tapko
//     Technologies GmbH; and
//   - subject to the license rights and restrictions in your commercial license for the software
//     from Tapko Technologies GmbH.

// * Any redistribution and use of any object code compiled from the source code and any
//   resulting derivative works, are licensed by TI:
//   - for use only with TI Devices and the Tapko KAIstack software;
//   - only if you have first obtained a commercial license for the software from Tapko
//     Technologies GmbH; and
//   - subject to the license rights and restrictions in your commercial license for the software
//     from Tapko Technologies GmbH.

// Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used
// to endorse or promote products derived from this software without specific prior written
// permission.

// DISCLAIMER.
// THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRTCT LIABILITY, OR TORT (INCLUDING NEGLTGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
// *****************************************************************************


#include <stdint.h>



//*****************************************************************************
// Application / HW IDs
//*****************************************************************************

//Defines the manufacturer ID for the application. 
//The manufacturer ID is a 16 bit value.
//The manufacturer ID is assigned by the KNX Association,
//@see ETS manufacturer tool
#define APP_MANUF_ID         KNX_MANUF_TAPKO


// Defines the application ID of the application. 
// The application ID is a 16 bit value. 
// This is together with the manufacturer ID and version a
// unique ID for the application over all manufacturers.
// The application ID is managed by each manufacturer itself.
// @see ETS manufacturer tool, APP_MANUF_ID, APP_APPL_VERSION
#if EIB_DEVICE_MODEL == EIB_DEVICE_MODEL_0705
  #define APP_APPL_ID          0xF022
#endif
#if EIB_DEVICE_MODEL == EIB_DEVICE_MODEL_2705
  #define APP_APPL_ID          0xF022
#endif
#if EIB_DEVICE_MODEL == EIB_DEVICE_MODEL_07B0 
  #define APP_APPL_ID          0xF023
#endif
#if EIB_DEVICE_MODEL == EIB_DEVICE_MODEL_27B0 
  #define APP_APPL_ID          0xF023
#endif


// Defines the application version. The application version is a 8 bit value 
// The application version is managed by each manufacturer itself 
// @see ETS manufacturer tool, APP_MANUF_ID, APP_APPL_ID
#define APP_APPL_VERSION     2


// Defines the order number of the hardware. 
// This constant is not evaluated by the system. 
// It can be displayed with ETS. 
// The order number is a 10 byte string 
// @see ETS manufacturer tool
#define APP_ORDER_NR         0,0,0,0,0,0,0,0,0,0 


// This constant defines the hardware type. 
// This type has to be 6 bytes long. 
// The usage and constraints are defined in the KNX specification. 
// The usage of the hardware type is enabled by the device model or via KNX_HW_TYPE_ENABLE. 
// The first byte of the hardware type should be the manufacturer ID.
#define KNX_HW_TYPE          _LOBYTE(APP_MANUF_ID),_HIBYTE(APP_MANUF_ID),0xff,0xff,0xff,0x01
#define KNX_FIRMWARE_VER     KNX_VERSION(2,0,0)



//*****************************************************************************
// Size of system tables
//*****************************************************************************

// Size of the address table (excluding physical address) 
// With this constant the amount of memory for the address table is reserved.
// The maximum allowed size depends on the device model. 
#define APP_addrTabSize      30       

// Size of the association table  
// With this constant the amount of memory for the association table is reserved. 
// The maximum size depends on the device model. 
#define APP_assocTabSize     30       


// Number of group communication objects. 
// The maximum size depends on the device model. 
// For the thermostat we have 2 outputs and 1 input, 
// so the total number of objects is 3
#define APP_objectTabSize    3        


//*****************************************************************************
// Timer Configuration
//*****************************************************************************

// Specifies the number of timers, which are available. 
// 3 Timer are used by the system. By default the timer numbers 0 to 2 are used.
#define TM_TIMER_NR          4         // number of timer (3 used by system)
#define UserTimer            3

//*****************************************************************************
// Communication Objects definitions
//*****************************************************************************
typedef enum {
#define KAI_DEF_CO_ENUM
#include "def_CommObj.h"
#include "cotab.h"
} CommObjects;


//*****************************************************************************
// Definition of BCU-RAM
//*****************************************************************************

// This structure contains the data of the group communication objects and RAM-Flags.
typedef struct {
  uint8_t ramFlags[APP_objectTabSize];              // RAM-flags
  uint8_t in0;                                      // object value: input
  uint8_t heat;                                     // object value: output
  uint8_t ac;                                     // object value: output
} APP_Ram;


//*****************************************************************************
// Definition of EEPROM-Parameter for ETS
//*****************************************************************************

// The parameters are defined in the structure APP_Parameter in app.h.
// The parameters are defined as a part of the structure DM_EEPROM, 
// which contains the whole EEPROM data. 
// At least one parameter has to be defined. 
// This structure can contain other structures or arrays.  
// The parameters are initialised in the file app_data.c.

typedef struct
{
  uint16_t delayTime;         
} APP_Parameter;


//*****************************************************************************
// Definition of EEPROM-Parameter not in ETS
//*****************************************************************************
typedef struct
{
  uint16_t dummy;
} APP_Eeprom;

