/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// #includes
//*****************************************************************************
#include "thermostat.h"
#include "Sharp96x96.h"
#include "HDC1000.h"
#include "i2c.h"
#include "rtc.h"
#include "msp430fr5969.h"

void UserButtonsInit(void);

//*****************************************************************************
// Init function definitions
//*****************************************************************************

//*****************************************************************************
//
// Initialize user buttons, LCD, I2C interface and HDC1000
//
//
// This function initializes thermostat front end
// 
// return None.
//
//*****************************************************************************
void System_init(void)
{
	//
	// Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    //
	PM5CTL0 &= ~LOCKLPM5;

 	// Initialize user buttons
	 UserButtonsInit();

	// Initialize RTC 
	RTC_init();
	
    // Initialize LCD driver and the context for the LCD
    Sharp96x96_initDisplay();
    GrContextInit(&sContext, &g_sharp96x96LCD);
    GrContextForegroundSet(&sContext, ClrBlack);
    GrContextBackgroundSet(&sContext, ClrWhite);

    //Setup USCI_B1 for I2C operation
    USCI_B_I2C_masterInit();
    //Initialize HDC1000
    HDC1000_init();

    //Do the first reading from HDC1000
    HDC1000_startMeasurement();

}

//*****************************************************************************
//
// Initialize user buttons
//
//
// This function initializes the 4 user buttons on the thermostat
// 
// return None.
//
//*****************************************************************************
void UserButtonsInit(void)
{
	//Button DOWN
	// Direction is input
	P3DIR &= ~BIT4;
	// Internal pull-up on P3.4
	P3OUT |= BIT4;
	// Enable pull-up resistor
	P3REN |= BIT4;
	// Lo/Hi edge
	P3IES &= ~BIT4; 
	// Clear all P3 interrupt flags                            
	P3IFG = 0;
	// interrupt enabled                                
    P3IE |= BIT4;                              

	//Button UP
	// Direction is input
	P3DIR &= ~BIT5;
	// Internal pull-up on P3.5
	P3OUT |= BIT5;
	// Enable pull-up resistor
	P3REN |= BIT5;
	// Lo/Hi edge
	P3IES &= ~BIT5; 
	// Clear all P3 interrupt flags                            
	P3IFG = 0;
	// interrupt enabled                                
    P3IE |= BIT5;  

	//Button Mode 1
	// Direction is input
	P3DIR &= ~BIT6;
	// Internal pull-up on P3.6
	P3OUT |= BIT6;
	// Enable pull-up resistor
	P3REN |= BIT6;
	// Lo/Hi edge
	P3IES &= ~BIT6; 
	// Clear all P3 interrupt flags                            
	P3IFG = 0;
	// interrupt enabled                                
    P3IE |= BIT6;  

	//Button Mode 2
	// Direction is input
	P3DIR &= ~BIT0;
	// Internal pull-up on P3.0
	P3OUT |= BIT0;
	// Enable pull-up resistor
	P3REN |= BIT0;
	// Lo/Hi edge
	P3IES &= ~BIT0; 
	// Clear all P3 interrupt flags                            
	P3IFG = 0;
	// interrupt enabled                                
    P3IE |= BIT0;  

}
