/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// #includes
//*****************************************************************************
#include "grlib_thermostat.h"

//*****************************************************************************
//
// Draws a horizontal line.
//
// param context is a pointer to the drawing context to use.
// param x1 is the X coordinate of one end of the line.
// param x2 is the X coordinate of the other end of the line.
// param y is the Y coordinate of the line.
//
// This function draws a horizontal line, taking advantage of the fact that
// the line is horizontal to draw it more efficiently.  The clipping of the
// horizontal line to the clipping rectangle is performed within this routine;
// the display driver's horizontal line routine is used to perform the actual
// line drawing.
//
// return None.
//
//*****************************************************************************
void Graphics_drawLineH(const Graphics_Context *context, int32_t x1, int32_t x2,
		int32_t  y)
{
    int32_t  temp;

    //
    // Check the arguments.
    //
    // assert(context);

    //
    // If the Y coordinate of this line is not in the clipping region, then
    // there is nothing to be done.
    //
    if((y < context->clipRegion.yMin) ||
       (y > context->clipRegion.yMax))
    {
        return;
    }

    //
    // Swap the X coordinates if the first is larger than the second.
    //
    if(x1 > x2)
    {
        temp = x1;
        x1 = x2;
        x2 = temp;
    }

    //
    // If the entire line is outside the clipping region, then there is nothing
    // to be done.
    //
    if((x1 > context->clipRegion.xMax) ||
       (x2 < context->clipRegion.xMin))
    {
        return;
    }

    //
    // Clip the starting coordinate to the left side of the clipping region if
    // required.
    //
    if(x1 < context->clipRegion.xMin)
    {
        x1 = context->clipRegion.xMin;
    }

    //
    // Clip the ending coordinate to the right side of the clipping region if
    // required.
    //
    if(x2 > context->clipRegion.xMax)
    {
        x2 = context->clipRegion.xMax;
    }

    //
    // Call the low level horizontal line drawing routine.
    //
    Graphics_drawHorizontalLineOnDisplay(context->display, x1, x2, y,
    		context->foreground);
}


