// *****************************************************************************
// FORM OF LICENSE FOR APPWIZARD FILES

// TEXAS INSTRUMENTS TEXT FILE LICENSE
// FOR TAPKO APPWIZARD FILES TO BE USED WITH TAPKO KAISTACK

// Copyright (c) 2015 Texas Instruments Incorporated
// Portions Copyright (c) 1999 * 2009 Tapko Technologies GmbH

// All rights reserved not granted herein.

// Limited License.

// Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive license under
// copyrights and patents it now or hereafter owns or controls to make, have made, use, import,
// offer to sell and sell ("Utilize") this software subject to the terms herein. With respect to the
// foregoing patent license, such license is granted solely to the extent that any such patent is
// necessary to Utilize the software alone. The patent license shall not apply to any combinations
// which include this software, other than combinations with devices manufactured by or for TI ("TI
// Devices"). No hardware patent is licensed hereunder'

// Use for Evaluation, Development and Support.

// You may copy, have copied, display and have displayed internally, distribute internally and use
// and have used internally this software for the sole purposes of evaluating and testing the
// software and designing, developing, maintaining and supporting your products, provided that
// the following conditions are met:

// * No reverse engineering, decompilation, or disassembly of this software is permitted with
//   respect to any software provided in binary form.
// * Any use is licensed by TI for use only with TI Devices and the Tapko KAIstack software.
// * Nothing shall obligate TI to provide you with source code for the software licensed and
//   provided to you in object code.

// Redistributions.

// Redistributions, if permitted as provided below, must preserve existing copyright notices and reproduce
// this license (including the above copyright notices and the disclaimer and (if
// applicable) source code license limitations below) in the documentation and/or other materials
// provided with the distribution.

// Redistributions in Binary Form.

// Redistribution in binary form, without modification, is permitted provided that the following
// conditions are met:
// * No reverse engineering, decompilation, or disassembly of this software is permitted with
//   respect to any software provided in binary form.
// * Any redistribution and use are licensed by TI:
//   - for use only with TI Devices and the Tapko KAIstack software;
//   - only if you have first obtained a commercial license for the software from Tapko
//     Technologies GmbH; and
//   - subject to the license rights and restrictions in your commercial license for the software
//     from Tapko Technologies GmbH.
// * Nothing shall obligate TI to provide you with source code for the software licensed and
// provided to you in object code.

// Redistributions in Source Form or Compiled from Source.

// If software source code is provided to you, modification and redistribution of the source code are
// permitted provided that the following conditions are met:

// * Any redistribution and use of the source code, including any resulting derivative works, are
//   licensed by TI:
//   - for use only with TI Devices and the Tapko KAIstack software;
//   - only if you have first obtained a commercial license for the software from Tapko
//     Technologies GmbH; and
//   - subject to the license rights and restrictions in your commercial license for the software
//     from Tapko Technologies GmbH.

// * Any redistribution and use of any object code compiled from the source code and any
//   resulting derivative works, are licensed by TI:
//   - for use only with TI Devices and the Tapko KAIstack software;
//   - only if you have first obtained a commercial license for the software from Tapko
//     Technologies GmbH; and
//   - subject to the license rights and restrictions in your commercial license for the software
//     from Tapko Technologies GmbH.

// Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used
// to endorse or promote products derived from this software without specific prior written
// permission.

// DISCLAIMER.
// THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRTCT LIABILITY, OR TORT (INCLUDING NEGLTGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
// *****************************************************************************


// ##Source Files
// src/app_data.c "app_data.c"        | contains the definition of the constant data (tables and parameter) of the application 
// src/thermostat.c "thermostat.c"    | contains the application code
// src/main.c "main.c"                | contains the main
// src/app.h "app.h"                  | contains the declaration of variables, constants ..
// src/cotab.h "cotab.h"              | contains the definition of the communication objects and addresses
// src/project.h "project.h"          | contains the settings of KAIstack
// @note Please set your target and device model in project.h.




//*****************************************************************************
// KNX #includes
//*****************************************************************************
#include "project.h"
#include "eib_stk.h"
#include "app.h"
#include EIB_DEVICE_MODEL_APP_H_FILE


//*****************************************************************************
// Applicaiton #includes
//*****************************************************************************
#include "thermostat.h"
#include "HDC1000.h"
#include "msp430fr5969.h"
#include "myTimer.h"
#include "string.h"

//*****************************************************************************
// #defines
//*****************************************************************************

// System status
#define SYSTEM_OFF     0                  // target_temperature = temperature
#define SYSTEM_AC      2                  // target_temperature > temperature
#define SYSTEM_HEAT    4                  // target_temperature < temperature

// Direction for buttons
#define NONE           0
#define UP             1
#define DOWN           2

// select menu options
#define SEL_RUN_CLOCK  2
#define SEL_SET_HOUR   4
#define SEL_SET_MIN    6
#define SEL_SET_DOW    8

// select_prog menu options
#define SET_TEMP       0
#define PROG_TEMP      2
#define PROG_HOUR      4
#define PROG_MIN       6
#define PROG_DUR_HOUR  8
#define PROG_DUR_MIN  10

// Mode - set time (CONFIG) or update the HOME/AWAY profiles (PROFILE)
#define PROFILE        0
#define CONFIG         1

// Thermostat profiles 
#define HOME           0
#define AWAY           1

// INFO memory addresses - preserve value after power down or reset
#define TARGET_TEMPERATURE_ADDR   0x1980
#define PROG_TEMPERATURE_ADDR     0x1981
#define PROG_HR_START_ADDR        0x1982
#define PROG_MIN_START_ADDR       0x1983
#define PROG_HR_END_ADDR          0x1984
#define PROG_MIN_END_ADDR         0x1985
#define PROFILE_ADDR              0x1986
#define HR_ADDR                   0x1987
#define MIN_ADDR                  0x1988
#define DOW_ADDR                  0x1989

//*****************************************************************************
// Local Variables
//*****************************************************************************
int32_t   HDC_Temp;                       // Stores HDC1000 raw temperature     
uint32_t  HDC_RH;                         // Stores HDC1000 raw humidity 
bool HDC_DRDY = 0;                        // Indicates HDC1000 measurement ready 
bool HDC_Start = 0;                       // Indicates new HDC1000 measurement needed 
uint8_t select = 0;                       // Keeps menu option for updating time and date - MODE1            
uint8_t select_prog = 0;                  // Keeps menu option for updating profiles - MODE2
uint8_t temperature = 0;                  // Temperature in degree C                                     
uint8_t humidity = 0;                     // Humidity in %
uint8_t target_temperature = 23;          // User selected temperature
uint8_t prog_temperature = 23;            // Programmed temperature for profile
uint8_t prog_hr_start = 12;               // Profile start time - hour  
uint8_t prog_min_start = 55;              // Profile start time - min              
uint8_t prog_hr_end = 0;                  // Profile end time - hour
uint8_t prog_min_end = 0;                 // Profile end time - min
bool value_valid = 1;                     // Only change profile once at start hr:min              
uint8_t system_status = 0;                // System Status: on, heat or AC
uint8_t system_status_new = 0;            // System Status change: on, heat or AC
uint8_t direction = NONE;                 // User button directions
uint8_t mode = PROFILE;                   // MODE can be update time (CONFIG) or profile (PROFILE)                 
uint8_t profile = HOME;                   // 2 profiles: HOME and AWAY. 



//*****************************************************************************
// Initialization of application variables
//*****************************************************************************
void APP_Init(void)
{ 
 // initalization of the application variables
 
 // initialize the object value
  OBJ_VALUE->in0 = 0;                  
 // Start running the clock with the thermostat
 select = SEL_RUN_CLOCK;
 
 // Read parameters from INFO memory 
 target_temperature = __data20_read_char(TARGET_TEMPERATURE_ADDR);
 prog_temperature = __data20_read_char(PROG_TEMPERATURE_ADDR);
 prog_hr_start = __data20_read_char(PROG_HR_START_ADDR);
 prog_min_start = __data20_read_char(PROG_MIN_START_ADDR);
 prog_hr_end = __data20_read_char(PROG_HR_END_ADDR);
 prog_min_end = __data20_read_char(PROG_MIN_END_ADDR);
 profile = __data20_read_char(PROFILE_ADDR); 

  // If device is blank then the time will be 0xFF (not a valid time). 
 if(prog_hr_start > 23)
  {
    prog_hr_start = 10;
   __data20_write_char(PROG_HR_START_ADDR, prog_hr_start);  
  }
   if(prog_min_start > 59)
  {
    prog_min_start = 0;
   __data20_write_char(PROG_MIN_START_ADDR, prog_min_start);  
  }
   if(prog_hr_end > 23)
  {
    prog_hr_end = 23;
   __data20_write_char(PROG_HR_END_ADDR, prog_hr_end);  
  }
   if(prog_min_end > 59)
  {
    prog_min_end = 59;
   __data20_write_char(PROG_MIN_END_ADDR, prog_min_end);  
  }

  // If device is blank then the time will be 0xFF (not a valid time). 
 if(__data20_read_char(HR_ADDR) > 23)
    {
      RTCHOUR = 0;
     __data20_write_char(HR_ADDR, RTCHOUR);  
    }
  else
    RTCHOUR = __data20_read_char(HR_ADDR);

   if(__data20_read_char(MIN_ADDR) > 59)
    {
      RTCMIN = 0;
     __data20_write_char(MIN_ADDR, RTCMIN);  
    }
    else 
      RTCMIN = __data20_read_char(MIN_ADDR); 

   if(__data20_read_char(DOW_ADDR) > 6)
    {
      RTCDOW = 0;
     __data20_write_char(DOW_ADDR, RTCDOW);  
    }
    else 
      RTCDOW = __data20_read_char(DOW_ADDR); 


      // Blank device
      if(profile == 0xFF){
        profile = HOME;
        __data20_write_char(PROFILE_ADDR, profile);
      }

      

}

//*****************************************************************************
// Main program of application (cyclic called)
//*****************************************************************************
void APP_Main(void)
{
  // Set data and time menu  or update screen with latest info (temp, humidity, profile, time, etc)
  switch (select) {
    // select = SEL_RUN_CLOCK is normal operation. Thermostat is running. Request new measurement from
    // sensor or read last measurement. If user is updating a profile, then update data on the LCD
    // Otherwise just update the screen to show the last time, temperature and humidity
    case SEL_RUN_CLOCK:
        // If new measurement is needed, request it from sensor
        if(HDC_Start)
          {
            HDC_Start = 0;
            HDC1000_startMeasurement();
          }

          //If a measurement is ready, read it. Set by nDRDY_HDC interrupt
          if (HDC_DRDY) 
          {
            //Read HDC1000 Measurement
            HDC1000_readMeasurement();
            //Reset HDC_DRDY
            HDC_DRDY = 0;
            // Set HDC_Start to begin a new measurement
            HDC_Start = 1;
          }

        // Update HOME or AWAY profile  
        switch(select_prog){
              // Update target temperature based on user input
              case SET_TEMP:
                target_temperature = updateParameter(target_temperature,50,0);
                __data20_write_char(TARGET_TEMPERATURE_ADDR, target_temperature);
                break;
              // Update target temperature of profile   
              case PROG_TEMP:
                prog_temperature = updateParameter(prog_temperature,50,0);
                __data20_write_char(PROG_TEMPERATURE_ADDR, prog_temperature);
                break;
              // Update start hour of profile  
              case PROG_HOUR:
                prog_hr_start = updateParameter(prog_hr_start,23,0);
                __data20_write_char(PROG_HR_START_ADDR, prog_hr_start);
                break;
              // Update start minute of profile  
              case PROG_MIN:
                prog_min_start = updateParameter(prog_min_start,59,0);
                __data20_write_char(PROG_MIN_START_ADDR, prog_min_start);
                break;
              // Update end hour of profile  
              case PROG_DUR_HOUR:
                prog_hr_end= updateParameter(prog_hr_end,23,0);
                __data20_write_char(PROG_HR_END_ADDR, prog_hr_end);
                break;
              // Update end minute of profile  
              case PROG_DUR_MIN:
                prog_min_end= updateParameter(prog_min_end,59,0);
                __data20_write_char(PROG_MIN_END_ADDR, prog_min_end);
                break;
              // User is not updating anything. Just update screen with latest time/temp/humidity  
              default:                
                LCD_update(1, 1);
                break;
                }        

          // If mode has not changed, then go into LMP3
          if(select == SEL_RUN_CLOCK )
          __bis_SR_register(LPM3_bits + GIE);
          break;

    // Update RTC hour - User input      
    case SEL_SET_HOUR:
        RTCHOUR = Clock_setParameter(RTCHOUR, 23, 0);
        __data20_write_char(HR_ADDR, RTCHOUR);
        break;
    // Update RTC minute - User input       
    case SEL_SET_MIN:
        RTCMIN = Clock_setParameter(RTCMIN, 59, 0);
        __data20_write_char(MIN_ADDR, RTCMIN);
        break;
    // Update RTC day of week - User input   
    case SEL_SET_DOW:
        RTCDOW = Clock_setParameter(RTCDOW, 6, 0);
        __data20_write_char(DOW_ADDR, RTCDOW);
        break;

    default: break;
    }

    // If the system status changed, then send a message over the KNX bus  
    if(system_status != system_status_new)
    { 
      // What is the new system status?     
      switch(system_status_new){
        // Turn heater on, AC off
        case SYSTEM_HEAT:
          OBJ_VALUE->heat = 1;
          OBJ_VALUE->ac = 0;
          SetTransmitRequest(CO_heat);
          SetTransmitRequest(CO_ac);
          break;
        // Turn heater off, AC on
        case SYSTEM_AC:
          OBJ_VALUE->heat = 0;
          OBJ_VALUE->ac = 1;
          SetTransmitRequest(CO_heat);
          SetTransmitRequest(CO_ac);
          break;
        // Turn heater and AC off  
        case SYSTEM_OFF:
          OBJ_VALUE->heat = 0;
          OBJ_VALUE->ac = 0;
          SetTransmitRequest(CO_heat);
          SetTransmitRequest(CO_ac);
          break;
        default: break;
      }
      // Update system_status to detect next change
      system_status = system_status_new;

    }

  // check if object value was received
  if (CheckUpdateFlag(CO_in0))         
  {
    // Update LED with the value received
     KSD_SwitchUserLED(OBJ_VALUE->in0, 1);
  }

}
#if SC_APPLICATION_TYPE == SC_APPL_CALL_EVENT

// Event handler for the application
void APP_EventHandler(SC_tEventMask event)
{
  if ((event & SC_EVENT_INIT) != 0)
    APP_Init();                        // Initialization of the application
  if ((event & ~SC_EVENT_INIT) != 0) 
    APP_Main();                        // application main routine
  
                                       // enable the next line for cyclic application call
  SC_SendEventAppl(SC_TASK_APP1, SC_EVENT_CYCLE);
}
#endif

//*****************************************************************************
// Save program of application 
//*****************************************************************************
void APP_Save(void)
{
// This function is not enabled in TIDM-KNXTHERMOSTAT
}

//*****************************************************************************
// NOTE: in APPHW... the group objects and Parameter may not be valid
//*****************************************************************************
void APPHW_Init(void)
{

// Ground P2.2 (RXD_DISKR_INV_EN) if bit-based interface is used
#if KNX_TARGET == MSP430_IAR5_DF
  P2DIR |= BIT2;
  P2OUT &= ~BIT2;
#endif

    // Init application LEDs. Defined in system_15\targets\TI\msp430_common\HandMadeEval_MSP430FR59xx.h
    KSD_InitUserLED();                   

    // Initialize application hardware
    System_init();     
}

void APPHW_Cycle(void)
{
}

#if SC_APPLICATION_TYPE == SC_APPL_CALL_EVENT

// Event handler for hardware part of the application
void APPHW_EventHandler(SC_tEventMask event)
{
  if ((event & SC_EVENT_INIT) != 0)
    APPHW_Init();                      // Initialization
  if ((event & ~SC_EVENT_INIT) != 0)
    APPHW_Cycle();                     // cycle

                                       // enable the next line for cyclic application call
  SC_SendEvent(SC_TASK_APPHW, SC_EVENT_CYCLE);
}
#endif
//*****************************************************************************
//
// Refresh LCD with latest data
//
//
// This function clears the LCD and then writes the latest value. 
// 
// param value is the parameter to be updated on the LCD 
// param blink is used to show or hide a specific parameter to create the blink
// effect
//
// return None
//
//*****************************************************************************
void LCD_update(uint8_t blink, uint16_t value)
{
    // Local variables of LCD_update
    char outString[32];
    char dataValue[10] = "";
    uint8_t hour, min, sec, weekday;
    uint8_t select_save, prog_save;
    char dow[10] = "";

    // Save menu variables in case of interrupt
    select_save = select;                   
    prog_save = select_prog;

    // Stop watchdog to update LCD. Else device will reset
    WDG_STOP();         

    // Set time variables to RTC register values
    hour = RTCHOUR; min = RTCMIN; sec = RTCSEC; weekday = RTCDOW;

    // Update memory with laterst time
    __data20_write_char(HR_ADDR, RTCHOUR); 
    __data20_write_char(MIN_ADDR, RTCMIN);  
    __data20_write_char(DOW_ADDR, RTCDOW); 

    // If in update time menu, overwrite RTC value for parameter being set
    switch (select_save) {  
      case  SEL_RUN_CLOCK:  
          // Update profile menu or current set at temperature if user has any input            
          switch (prog_save) {
          case SET_TEMP:
              target_temperature = value;
              break;
            case PROG_TEMP:
              prog_temperature = value;
              break;
            case PROG_HOUR:
              prog_hr_start = value;
              break;
            case PROG_MIN:
              prog_min_start = value;
              break;
            case PROG_DUR_HOUR:
              prog_hr_end= value;
              break;
            case PROG_DUR_MIN:
              prog_min_end= value;
              break;
            default:
              break;
          }
      break;
      // Set hour 
      case  SEL_SET_HOUR:                
          hour = value;
          break;
      // Set minute 
      case  SEL_SET_MIN: 
          min = value;
          break;
      // Setting day of week
      case  SEL_SET_DOW: 
          weekday = value;
          break;

      default: break;
    }

    // Decode day of week
    switch (weekday) { 
      case 0:
          strcpy(dow, "SUN");
          break;
      case 1:
          strcpy(dow, "MON");
          break;
      case 2:
          strcpy(dow, "TU");
          break;
      case 3:
          strcpy(dow, "WED");
          break;
      case 4:
          strcpy(dow, "TH");
          break;
      case 5:
          strcpy(dow, "FRI");
          break;
      case 6:
          strcpy(dow, "SAT");
          break;
      default: break;
    }

    // Check to see if it's time to change to another profile
    uint16_t local_target;

    // Change profiles if it is time. Must swap start and end times as well as the profile variable
    if((hour == prog_hr_start) && (min == prog_min_start) && (prog_save == SET_TEMP)){
      // value is no longer valid so target_temperature needs to be updated
      value_valid = 0;

      // Swap start and end hour      
      local_target = prog_hr_end;
      prog_hr_end = prog_hr_start;
      prog_hr_start = local_target;

      // Swap start and end minute 
      local_target = prog_min_end;
      prog_min_end = prog_min_start;
      prog_min_start = local_target;

      // Start and end cannot be the same. Add 1 minute
      if((prog_hr_end == prog_hr_start) && (prog_min_end == prog_min_start))  
      {
        prog_min_start += 1;
        if(prog_min_start == 60)
        {
          prog_min_start = 0;
          prog_hr_start +=1;
          if(prog_hr_start == 24)
          {
            prog_hr_start = 0;
          }
        }
      }

      // Swap profiles
      if(profile == HOME){
        profile = AWAY;
      }
      else
      {
        profile = HOME;
      }

      __data20_write_char(PROFILE_ADDR, profile);

    }

    // Begin screen update
    GrClearDisplay(&sContext);
    GrContextForegroundSet(&sContext, ClrBlack);

    // Write Day of Week - if setting DOW, blink it
    if ((select_save != SEL_SET_DOW) || (blink == 1)) { 
        GrContextFontSet(&sContext, &g_sFontCmss12);
        GrStringDraw(&sContext, (int8_t*)dow, -1, 3, 0, 0);
    }

    // Write time
    // Clear dataValue
    strcpy(dataValue, "");  
    // write hour value
    if (hour >= 10) {       
        convertASCIItoUnsignedInt(hour, dataValue);
        strcpy(outString, dataValue);
    }
    else {
        strcpy(outString, "");
        dataValue[0] = 0x30;
        dataValue[1] = hour % 10 + 0x30;
        strncat(outString, dataValue, 2);
    }
    // append colon between hour and minute
    strcat(outString, ":");
    // append minute value to time if it is not blinking                     
    if ((select_save != SEL_SET_MIN) || (blink == 1)) {   
      if (min >= 10) {                      
        convertASCIItoUnsignedInt(min, dataValue);
        strncat(outString, dataValue, 2);
      }
      else {
        dataValue[0] = 0x30;
        dataValue[1] = min % 10 + 0x30;
        strncat(outString, dataValue, 2);
      }
    }

   // Write seconds if not setting time
   if(select_save == SEL_RUN_CLOCK){
    if (sec >= 10) {
        convertASCIItoUnsignedInt(sec, dataValue);
        // append colon between minute and second
        strcat(outString, ":");                     
        strncat(outString, dataValue, 2);

    }
    else {
      strcat(outString, "");
      // append colon between minute and second
      strcat(outString, ":");                     
      dataValue[0] = 0x30;
      dataValue[1] = sec % 10 + 0x30;
      strncat(outString, dataValue, 2);
    }
   }
    GrContextFontSet(&sContext, &g_sFontCmss12);
    // write time (hr:min:sec)
    GrStringDraw(&sContext, (int8_t*)outString, -1, 56, 0, 0); 


    // if blinking hour, overwrite it with background color
    if ((select_save == SEL_SET_HOUR) && (blink == 0)) {
      GrContextForegroundSet(&sContext, ClrWhite);
      strcpy(dataValue, ""); 
      if (hour >= 10) {
          convertASCIItoUnsignedInt(hour, dataValue);
          strcpy(outString, dataValue);
      }
      else {
          strcpy(outString, "");
          dataValue[0] = 0x30;
          dataValue[1] = hour % 10 + 0x30;
          strncat(outString, dataValue, 2);
      }
      GrStringDraw(&sContext, (int8_t*)outString, -1, 56, 0, 0);
      GrContextForegroundSet(&sContext, ClrBlack);
    }
    
    int i;
    // Draw dark rectable
    for(i=11;i<42;i++){
    Graphics_drawLineH(&sContext, 0, 96, i);
    }

    // Write current temperature
    GrContextForegroundSet(&sContext, ClrWhite);
    temperature = (int16_t)HDC_Temp;
    humidity = (int16_t)HDC_RH;
    strcpy(dataValue, ""); // Clear dataValue
    convertASCIItoUnsignedInt(temperature, dataValue);
    strcpy(outString, dataValue);
    GrContextFontSet(&sContext, &g_sFontCmss20b);
    GrStringDraw(&sContext, (int8_t*)outString, -1, 35, 13, 0);

    // Based on temperature value, change X position of pixels. 
    uint16_t temp;
    if(temperature < 10)
      temp = 50;
    else if (temperature < 100)
      temp = 57;
    else
      temp = 64;

    
    // Write degree C
    strcpy(dataValue, ""); 
    strcpy(outString, "o");
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, temp, 10, 0);

    strcpy(outString, ""); 
    strcpy(outString, "C");
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, temp+6, 14, 0);

    // Write current humidity
    strcpy(dataValue, "");
    strcpy(outString, "Humidity: ");
    convertASCIItoUnsignedInt(humidity, dataValue);
    strncat(outString, dataValue, 2);
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, 15, 28, 0);

    // Do percentage sign for humidity
    strcpy(dataValue, ""); 
    strcpy(outString, "o");
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, 75, 24, 0);

    strcpy(dataValue, ""); 
    strcpy(outString, "/");
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, 79, 28, 0);

    strcpy(dataValue, ""); 
    strcpy(outString, "o");
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, 82, 30, 0);

    GrContextForegroundSet(&sContext, ClrBlack);

    // Write profile menu if user has pressed MODE2
    if(prog_save != SET_TEMP){
      strcpy(dataValue, ""); 
      if(profile == AWAY)
      strcpy(outString, "Home mode: ");
      else
      strcpy(outString, "Away mode: ");
      GrContextFontSet(&sContext, &g_sFontCmss12);
      GrStringDraw(&sContext, (int8_t*)outString, -1, 2, 42, 0);

      strcpy(dataValue, ""); 
      convertASCIItoUnsignedInt(prog_temperature, dataValue);
      strcpy(outString, dataValue);
      GrStringDraw(&sContext, (int8_t*)outString, -1, 65, 42, 0);

      // If blinking overwrite programmed temperature
      if ((select_prog == PROG_TEMP) && (blink == 0)) {
          GrContextForegroundSet(&sContext, ClrWhite);
          convertASCIItoUnsignedInt(prog_temperature, dataValue);
          strcpy(outString, dataValue);
          GrStringDraw(&sContext, (int8_t*)outString, -1, 65, 42, 0);
          GrContextForegroundSet(&sContext, ClrBlack);
      }

      if(prog_temperature < 10)
        temp = 75;
      else if (prog_temperature < 100)
        temp = 79;
      else
        temp = 84;

      // Write degree C
      strcpy(dataValue, ""); 
      strcpy(outString, "o");
      GrContextFontSet(&sContext, &g_sFontCmss12);
      GrStringDraw(&sContext, (int8_t*)outString, -1, temp, 39, 0);
      strcpy(outString, ""); 
      strcpy(outString, "C");
      GrContextFontSet(&sContext, &g_sFontCmss12);
      GrStringDraw(&sContext, (int8_t*)outString, -1, temp+6, 42, 0);

      // Write Start time
      strcpy(dataValue, ""); 
      strcpy(outString, "Start: ");
      GrContextFontSet(&sContext, &g_sFontCmss12);
      GrStringDraw(&sContext, (int8_t*)outString, -1, 2, 53, 0);

      // Write programming time - hour
      strcpy(dataValue, "");


      if (prog_hr_start >= 10) {      
          convertASCIItoUnsignedInt(prog_hr_start, dataValue);
          strcpy(outString, dataValue);
      }
      else {
          strcpy(outString, "");
          dataValue[0] = 0x30;
          dataValue[1] = prog_hr_start % 10 + 0x30;
          strncat(outString, dataValue, 2);
      }
      strcat(outString, ":");
      // append minute value to time  if it is not blinking
      if ((select_prog != 6) || (blink == 1)) {  
          if (prog_min_start >= 10) {                        
              convertASCIItoUnsignedInt(prog_min_start, dataValue);
              strncat(outString, dataValue, 2);
          }
          else {
              dataValue[0] = 0x30;
              dataValue[1] = prog_min_start % 10 + 0x30;
              strncat(outString, dataValue, 2);
          }
      }

      GrContextFontSet(&sContext, &g_sFontCmss12);
      GrStringDraw(&sContext, (int8_t*)outString, -1, 30, 53, 0);

      // If blinking overwrite hr
      if ((select_prog == PROG_HOUR) && (blink == 0)) {
          GrContextForegroundSet(&sContext, ClrWhite);
          if (prog_hr_start >= 10) {       // write hour value
              convertASCIItoUnsignedInt(prog_hr_start, dataValue);
              strcpy(outString, dataValue);
          }
          else {
              strcpy(outString, "");
              dataValue[0] = 0x30;
              dataValue[1] = prog_hr_start % 10 + 0x30;
              strncat(outString, dataValue, 2);
          }
          GrStringDraw(&sContext, (int8_t*)outString, -1, 30, 53, 0);
          GrContextForegroundSet(&sContext, ClrBlack);
      }

      // Write End time
      strcpy(dataValue, "");
      strcpy(outString, "End: ");
      GrContextFontSet(&sContext, &g_sFontCmss12);
      GrStringDraw(&sContext, (int8_t*)outString, -1, 2, 64, 0);

      // Write programming time - hour
      strcpy(dataValue, "");  
      if (prog_hr_end >= 10) {       
          convertASCIItoUnsignedInt(prog_hr_end, dataValue);
          strcpy(outString, dataValue);
      }
      else {
          strcpy(outString, "");
          dataValue[0] = 0x30;
          dataValue[1] = prog_hr_end % 10 + 0x30;
          strncat(outString, dataValue, 2);
      }
      strcat(outString, ":");
      // Append minute value to time if it is not blinking
      if ((select_prog != 10) || (blink == 1)) {   
          if (prog_min_end >= 10) {                      
              convertASCIItoUnsignedInt(prog_min_end, dataValue);
              strncat(outString, dataValue, 2);
          }
          else {
              dataValue[0] = 0x30;
              dataValue[1] = prog_min_end % 10 + 0x30;
              strncat(outString, dataValue, 2);
          }
      }
   
      GrContextFontSet(&sContext, &g_sFontCmss12);
      GrStringDraw(&sContext, (int8_t*)outString, -1, 23, 64, 0);

      // If blinking overwrite hr
      if ((select_prog == PROG_DUR_HOUR) && (blink == 0)) {
          GrContextForegroundSet(&sContext, ClrWhite);
          if (prog_hr_end >= 10) {       // write hour value
              convertASCIItoUnsignedInt(prog_hr_end, dataValue);
              strcpy(outString, dataValue);
          }
          else {
              strcpy(outString, "");
              dataValue[0] = 0x30;
              dataValue[1] = prog_hr_end % 10 + 0x30;
              strncat(outString, dataValue, 2);
          }
          GrStringDraw(&sContext, (int8_t*)outString, -1, 23, 64, 0);
          GrContextForegroundSet(&sContext, ClrBlack);
        }
      } // end of if(prog_save == SET_TEMP)

    // If user not setting profile, write current profile name to LCD  
    else{
        strcpy(dataValue, ""); 
        if(profile == AWAY)
        strcpy(outString, "Away");
        else
        strcpy(outString, "Home");
        GrContextFontSet(&sContext, &g_sFontCmss20b);
        GrStringDraw(&sContext, (int8_t*)outString, -1, 22, 42, 0);
        strcpy(dataValue, ""); 
        strcpy(outString, "Mode2 to schedule");
        GrContextFontSet(&sContext, &g_sFontCmss12);
        GrStringDraw(&sContext, (int8_t*)outString, -1, 2, 62, 0);
    }

    // Draw dark rectable
    for(i=75;i<96;i++){
    Graphics_drawLineH(&sContext, 0, 96, i);
    }
    
    // Write current set temperature
    GrContextForegroundSet(&sContext, ClrWhite);
    strcpy(dataValue, ""); 
    strcpy(outString, "Set at ");
    convertASCIItoUnsignedInt(target_temperature, dataValue);
    strncat(outString, dataValue, 3);
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, 2, 75, 0);


    if(target_temperature < 10)
      temp = 45;
    else if (target_temperature < 100)
      temp = 52;
    else
      temp = 57;

    // Write degree C
    strcpy(dataValue, ""); 
    strcpy(outString, "o");
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, temp, 72, 0);

    strcpy(outString, ""); 
    strcpy(outString, "C");
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, temp+6, 75, 0);

    // Write System Status
    strcpy(dataValue, ""); 
    if(target_temperature > temperature)
    {
      strcpy(outString, "Heat ON");
      // Update system_status_new to send message over the KNX bus
      system_status_new = SYSTEM_HEAT;
    }
    else if(target_temperature < temperature)
    {
      strcpy(outString, "AC ON");
      // Update system_status_new to send message over the KNX bus
      system_status_new = SYSTEM_AC;
    }
    else
    {
      strcpy(outString, "System OFF");
      // Update system_status_new to send message over the KNX bus
      system_status_new = SYSTEM_OFF;
    }
    strncat(outString, dataValue, 2);
    GrContextFontSet(&sContext, &g_sFontCmss12);
    GrStringDraw(&sContext, (int8_t*)outString, -1, 2, 85, 0);
    GrContextForegroundSet(&sContext, ClrBlack);
    GrFlush(&sContext);

    // Done writing to LCD. Restart WDT
    WDG_START();
    WDG_RESET();
}

//*****************************************************************************
//
// Convert Integer to ASCII
//
//
// This function clears the LCD and then writes the latest value. 
// 
// param hex is the integer to be converted
// param ASCII is the converted value
// effect
//
// return ASCII value
//
//*****************************************************************************
uint16_t convertASCIItoUnsignedInt(uint16_t hex, char *ASCII)
{
    uint16_t i, j;
    uint8_t flipped[5] = { 0, 0, 0, 0, 0 };

    // Convert from hex to ASCII
    for (i = 1; i < 6; i++) {
        flipped[i] = hex % 10 + 0x30;
        hex /= 10;
        if (hex == 0) {
            break;
        }
    }

    // Flip data values to correct position in array
    for (j = 0; j < 5; j++) {
        ASCII[j] = flipped[i - j];
    }

    return i;
}


//*****************************************************************************
//
// Update profile parameters
//
//
// This function updates parameters for the two profiles (HOME and AWAY)
// 
// param value is the parameter to be updated. 
// param max is the maximum value of 'value'
// param min is the minimum value of 'value'
//
// return updated value
//
//*****************************************************************************
uint16_t updateParameter(uint16_t value, uint16_t max, uint16_t min)
{
  // local variables for function updateParameter
  uint8_t temp0 = 0;  
  uint8_t blink = 1;

  // Store menu option 
  temp0 = select_prog;

  // Loop updating value until user changes the menu option. This allows the blinking of 
  // the value being updated on the LCD
  while ((select_prog == temp0)&&(mode == PROFILE)) {

    // User pressed the the 'UP' option. Direction is set in the IO ISR. 
    // Increase value by 1 and check if the maximum has been reached. Clear direction 
    // so that it does not increase again until user presses the UP button     
    if (direction == UP) {                    
      direction = NONE;
        value++;
        if (value > max) {                      
            value = min;
        }
    }

    // User pressed the the 'DOWN' option. Direction is set in the IO ISR. 
    // Decrease value by 1 and check if the minimum has been reached. Clear direction 
    // so that it does not decrease again until user presses the DOWN button     
    else if (direction == DOWN) {               
      direction = NONE;
        value--;
        if ((value < min) || (value > max)) {
            value = max;
      }
    }

    // Update LCD with latest value
    LCD_update(blink, value);

    // Toggle blink variable so that value blinks on LCD
    blink ^= 1;

    // Last option on menu reached. Change temp0 to break out of while loop
    if(select_prog == SET_TEMP)
    {
      temp0 = 0xFF;
    }
  }
    // Check to see if it's time to change to another profile
    uint16_t local_target;

    // Change target temperature if it is time to change profiles
    // value_valid is used to update other profile parameters 
    if(value_valid == 0){
      value_valid = 1;
      local_target = value;
      value = prog_temperature;
      prog_temperature = local_target;
    }

    return value;
}

//*****************************************************************************
//
// Update clock (RTC) parameters
//
//
// This function updates the RTC parameters
// 
// param value is the parameter to be updated. 
// param max is the maximum value of 'value'
// param min is the minimum value of 'value'
//
// return updated value
//
//*****************************************************************************
static uint8_t Clock_setParameter(uint8_t value, uint8_t max, uint8_t min)
{
    // local variables for function Clock_setParameter
    uint8_t temp0 = 0;
    uint8_t blink = 1;

    // Store menu option 
    temp0 = select;

    // Loop updating value until user changes the menu option. This allows the blinking of 
    // the value being updated on the LCD
    while ((select == temp0) && (mode == CONFIG)) {

        if (blink) {
            // User pressed the the 'UP' option. Direction is set in the IO ISR. 
            // Increase value by 1 and check if the maximum has been reached. Clear direction 
            // so that it does not increase again until user presses the UP button                                    
            if (direction == UP) {                 
              direction = NONE;
                value++;
                if (value > max) {                     
                    value = min;
                }
            }

            // User pressed the the 'UP' option. Direction is set in the IO ISR. 
            // Increase value by 1 and check if the maximum has been reached. Clear direction 
            // so that it does not increase again until user presses the UP button   
            else if (direction == DOWN) {               
              direction = NONE;
                value--;
                if ((value < min) || (value > max)) {
                    value = max;
                }
            }
        }

        // Update LCD with latest value
        LCD_update(blink, value);

        // Toggle blink variable so that value blinks on LCD
        blink ^= 1;
    }
    return value;
}



//*****************************************************************************
// Interrupt Service Routine - Port 1
//*****************************************************************************
#pragma vector=PORT1_VECTOR
__interrupt void Port_1(void)
{
    switch (__even_in_range(P1IV, P1IV_P1IFG7)) {

    case P1IV_NONE:   break;                    //No Interrupt pending 
    case P1IV_P1IFG0: break;                    //P1IV P1IFG.0 
    case P1IV_P1IFG1: break;                    //P1IV P1IFG.1
    case P1IV_P1IFG2: break;                    //P1IV P1IFG.2 
    case P1IV_P1IFG3:                           //P1IV P1IFG.3 - HDC_DRDY
      // Clear P1.0 IFG   
      P1IFG &= ~BIT3;   
      // Set HDC_DRDY to indicate data from HDC1000 is ready to be read        
      HDC_DRDY = 1;
      break;

    case P1IV_P1IFG4: break;                    //P1IV P1IFG.4
    case P1IV_P1IFG5: break;                    //P1IV P1IFG.5
    case P1IV_P1IFG6: break;                    //P1IV P1IFG.6
    case P1IV_P1IFG7: break;                    //P1IV P1IFG.7
    }

}


//*****************************************************************************
// Interrupt Service Routine - Port 3
//*****************************************************************************
#pragma vector=PORT3_VECTOR
__interrupt void Port_3(void)
{
    switch (__even_in_range(P3IV, P3IV_P3IFG7)) {

    case P3IV_NONE:   break;                    //No Interrupt pending 
    case P3IV_P3IFG0:                           //P3IV P3IFG.0 - MODE2
      // Clear P3.0 IFG
      P3IFG &= ~BIT0; 
      // P3.0 interrupt disable                           
      P3IE &= ~BIT0;
      // Debounce switch                              
      __delay_cycles(9000);     
      // Clear P3.0 IFG         
      P3IFG &= ~BIT0;
      // Disable interrupt on P3.0           
      P3IE &= ~BIT0;

      // Increase select_prog to go to the next menu option 
      select_prog+=2;

      // Restart RTC and switch to profile setting
      select = SEL_RUN_CLOCK;
      mode = PROFILE;
      RTCPS1CTL |= RT1PSIE + RT1IP_5;
      RTCCTL01 |= RTCTEVIE;
      RTCCTL01_H &= ~RTCHOLD_H;

      // Check to see if last option has been reached on menu and cycle to first
      if (select_prog > PROG_DUR_MIN)
      {
        select_prog = 0;
      }

      // Re-enable interrupts and exit active
      P3IFG &= ~BIT0;
      P3IE |= BIT0;
      __bic_SR_register_on_exit(LPM3_bits);
      break;
    case P3IV_P3IFG1: break;                           //P3IV P3IFG.1 
    case P3IV_P3IFG2: break;                           //P3IV P3IFG.2
    case P3IV_P3IFG3: break;                           //P3IV P3IFG.3 
    case P3IV_P3IFG4:                                  //P3IV P3IFG.4 - DONW
      // Clear P3.4 IFG
      P3IFG &= ~BIT4;
      // P3.4 interrupt disable  
      P3IE &= ~BIT4;
      // Set direction to DOWN
      direction = DOWN;

      // Re-enable interrupts and exit active
      P3IFG &= ~BIT4;
      P3IE |= BIT4;
      __bic_SR_register_on_exit(LPM3_bits);
      break;

    case P3IV_P3IFG5:                                  //P3IV P3IFG.5 - UP
      // Clear P3.5 IFG
      P3IFG &= ~BIT5;
      // P3.5 interrupt disable
      P3IE &= ~BIT5;
      // Set direction to UP
      direction = UP;

      // Re-enable interrupts and exit active
      P3IFG &= ~BIT5;
      P3IE |= BIT5;
      __bic_SR_register_on_exit(LPM3_bits);
      break;

    case P3IV_P3IFG6:                                  //P3IV P3IFG.6 - MODE1
      // Clear P3.6 IFG
      P3IFG &= ~BIT6; 
      // P3.6 interrupt disable          
      P3IE &= ~BIT6;
      // Debounce Switch                        
      __delay_cycles(9000);         

      // Set MODE to config
      mode = CONFIG;

      // Make sure the MODE2 is in default value of SET_TEMP
      select_prog = SET_TEMP;

      // stop clock and start setting parameters
      if (select == SEL_RUN_CLOCK) {                
        RTCCTL01_H |= RTCHOLD_H;
        RTCSEC = 0;
      }
      // Increase menu option
      select += 2;
      // last parameter set, start clock
      if (select > SEL_SET_DOW) {                 
          select = SEL_RUN_CLOCK;
          mode = PROFILE;
          // Reset seconds
          RTCSEC = 0;                     
          RTCPS1CTL |= RT1PSIE + RT1IP_5;
          RTCCTL01 |= RTCTEVIE;
          // Start running RTC 
          RTCCTL01_H &= ~RTCHOLD_H;
      }

      // Re-enable interrupts and exit active
      P3IFG &= ~BIT6;
      P3IE |= BIT6;
      __bic_SR_register_on_exit(LPM3_bits);
      break;

    case P3IV_P3IFG7: break;                           //P3IV P3IFG.7 
    }



}




